--beginvalidatingquery
	if exists (select * from dbo.sysobjects where id = object_id(N'[dbo].[sp_DatabaseVersion]') and OBJECTPROPERTY(id, N'IsProcedure') = 1)
    begin
            declare @ver int
            exec @ver=sp_DatabaseVersion
            if (@ver >= 7072)
				select 0, 'Already correct database version'
            else if (@ver = 7071)
                 select 1, 'Upgrading database'
            else
                 select -1, 'Invalid database version detected'
    end
    else
            select -1, 'Not an EPiServer database'
--endvalidatingquery
GO


PRINT N'Altering Table [dbo].[tblMappedIdentity]...';


GO
ALTER TABLE [dbo].[tblMappedIdentity]
    ADD [Metadata] NVARCHAR (MAX) NULL,
        [Saved]    DATETIME       NULL;


GO
PRINT N'Altering Procedure [dbo].[netMappedIdentityForProvider]...';


GO
ALTER PROCEDURE [dbo].[netMappedIdentityForProvider]
	@Provider NVARCHAR(255),
    @Saved datetime = NULL
AS
BEGIN
	SET NOCOUNT ON;

    IF @Saved IS NULL
    BEGIN
	    SELECT MI.pkID AS ContentId, MI.Provider, MI.ProviderUniqueId, MI.ContentGuid, MI.ExistingContentId, MI.ExistingCustomProvider, MI.Metadata, MI.Saved
	    FROM tblMappedIdentity AS MI
	    WHERE MI.Provider = @Provider
    END
    ELSE
    BEGIN
        SELECT MI.pkID AS ContentId, MI.Provider, MI.ProviderUniqueId, MI.ContentGuid, MI.ExistingContentId, MI.ExistingCustomProvider, MI.Metadata, MI.Saved
	    FROM tblMappedIdentity AS MI
	    WHERE MI.Provider = @Provider AND MI.Saved > @Saved
    END
END
GO
PRINT N'Altering Procedure [dbo].[netMappedIdentityGetByGuid]...';


GO
ALTER PROCEDURE [dbo].[netMappedIdentityGetByGuid]
	@ContentGuids dbo.GuidParameterTable READONLY
AS
BEGIN
	SET NOCOUNT ON;

	SELECT MI.pkID AS ContentId, MI.Provider, MI.ProviderUniqueId, MI.ContentGuid, MI.ExistingContentId, MI.ExistingCustomProvider, MI.Metadata, MI.Saved
	FROM tblMappedIdentity AS MI INNER JOIN @ContentGuids AS EI ON MI.ContentGuid = EI.Id
END
GO
PRINT N'Altering Procedure [dbo].[netMappedIdentityGetById]...';


GO
ALTER PROCEDURE [dbo].[netMappedIdentityGetById]
	@InternalIds dbo.ContentReferenceTable READONLY
AS
BEGIN
	SET NOCOUNT ON;

	SELECT MI.pkID AS ContentId, MI.Provider, MI.ProviderUniqueId, MI.ContentGuid, MI.ExistingContentId, MI.ExistingCustomProvider, MI.Metadata, MI.Saved
	FROM tblMappedIdentity AS MI 
	INNER JOIN @InternalIds AS EI ON (MI.pkID = EI.ID AND MI.Provider = EI.Provider)
	UNION (SELECT MI2.pkID AS ContentId, MI2.Provider, MI2.ProviderUniqueId, MI2.ContentGuid, MI2.ExistingContentId, MI2.ExistingCustomProvider, MI2.Metadata, MI2.Saved
		FROM tblMappedIdentity AS MI2
		INNER JOIN @InternalIds AS EI2 ON (MI2.ExistingContentId = EI2.ID)
		WHERE ((MI2.ExistingCustomProvider = 1 AND MI2.Provider = EI2.Provider) OR (MI2.ExistingCustomProvider IS NULL AND EI2.Provider IS NULL)))
	END
GO
PRINT N'Altering Procedure [dbo].[netMappedIdentityGetOrCreate]...';


GO
ALTER PROCEDURE [dbo].[netMappedIdentityGetOrCreate]
	@ExternalIds dbo.UriPartsTable READONLY,
	@CreateIfMissing BIT,
    @Saved datetime
AS
BEGIN
	SET NOCOUNT ON;

	--Create first missing entries
	IF @CreateIfMissing = 1
	BEGIN
		MERGE tblMappedIdentity AS Target
		USING @ExternalIds AS Source
		ON (Target.Provider = Source.Host AND Target.ProviderUniqueId = Source.Path)
		WHEN NOT MATCHED BY Target THEN
			INSERT (Provider, ProviderUniqueId, Saved)
			VALUES (Source.Host, Source.Path, @Saved);
	END

	SELECT MI.pkID AS ContentId, MI.Provider, MI.ProviderUniqueId, MI.ContentGuid, MI.ExistingContentId, MI.ExistingCustomProvider, MI.Metadata, MI.Saved
	FROM tblMappedIdentity AS MI INNER JOIN @ExternalIds AS EI ON MI.ProviderUniqueId = EI.Path
	WHERE MI.Provider = EI.Host
END
GO
PRINT N'Altering Procedure [dbo].[sp_DatabaseVersion]...';


GO
ALTER PROCEDURE [dbo].[sp_DatabaseVersion]
AS
	RETURN 7072
GO
PRINT N'Creating Procedure [dbo].[netMappedIdentitySetMetadata]...';


GO
CREATE PROCEDURE [dbo].[netMappedIdentitySetMetadata]
	@ContentGuid uniqueidentifier,
	@Metadata NVARCHAR(MAX),
    @Saved datetime
AS
BEGIN
	SET NOCOUNT ON;

	UPDATE
        tblMappedIdentity
    SET
        Metadata = @Metadata,
        Saved = @Saved
    WHERE
        ContentGuid = @ContentGuid

    SELECT pkID AS ContentId, Provider, ProviderUniqueId, ContentGuid, ExistingContentId, ExistingCustomProvider, Metadata, Saved
	FROM tblMappedIdentity WHERE ContentGuid = @ContentGuid
END
GO

PRINT N'Update complete.';


GO
